﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace AllControlsSamples.Controls
{
    [TemplateVisualState(GroupName = "CustomStates", Name = "StateOne")]
    [TemplateVisualState(GroupName = "CustomStates", Name = "StateTwo")]
    [TemplateVisualState(GroupName = "CustomStates", Name = "StateThree")]
    [TemplatePart(Name = "BackgroundPart", Type = typeof(FrameworkElement))]
    public class MyControl : Control
    {
        public MyControl()
        {
            this.DefaultStyleKey = typeof(MyControl);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            var background = GetTemplateChild("BackgroundPart") as FrameworkElement;
            if (background != null)
            {
                background.MouseLeftButtonDown += (s, e) =>
                {
                    var groups 
                        = VisualStateManager.GetVisualStateGroups(background);

                    if (groups == null
                        || groups.Count == 0)
                    {
                        return;
                    }

                    VisualStateGroup group 
                        = groups[0] as VisualStateGroup;

                    if (group == null)
                    {
                        return;
                    }

                    VisualState currentState = group.CurrentState;

                    switch (currentState.Name)
                    {
                        case "StateOne":
                            VisualStateManager.GoToState(this, "StateTwo", true);
                            break;
                        case "StateTwo":
                            VisualStateManager.GoToState(this, "StateThree", true);
                            break;
                        case "StateThree":
                            VisualStateManager.GoToState(this, "StateOne", true);
                            break;
                    }
                };

                VisualStateManager.GoToState(this, "StateOne", false);
            }
        }
    }
}
