﻿using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

namespace AttachedBehaviour.Extensions
{
    public class ElementRotator : DependencyObject
    {
        public const string InitialAnglePropertyName = "InitialAngle";

        public static double GetInitialAngle(DependencyObject obj)
        {
            return (double)obj.GetValue(InitialAngleProperty);
        }

        public static void SetInitialAngle(DependencyObject obj, double value)
        {
            obj.SetValue(InitialAngleProperty, value);
        }

        public static readonly DependencyProperty InitialAngleProperty 
            = DependencyProperty.RegisterAttached(
            InitialAnglePropertyName,
            typeof(double),
            typeof(ElementRotator),
            new PropertyMetadata(400.0, AttachToElement));

        private static void AttachToElement(
            DependencyObject d, 
            DependencyPropertyChangedEventArgs e)
        {
            var element = d as UIElement;

            if (element == null)
            {
                return;
            }

            if (e.NewValue == null)
            {
                Detach(element);
            }

            double initialAngle = (double)e.OldValue;

            if (initialAngle >= 360.0)
            {
                // The element was not initialized yet.
                // Attach the event handler.
                element.MouseLeftButtonDown += RotateElement;
                element.RenderTransformOrigin = new Point(0.5, 0.5);
            }

            element.RenderTransform = new RotateTransform
            {
                Angle = (double)e.NewValue
            };
        }

        private static void RotateElement(object s, MouseButtonEventArgs e)
        {
            var sender = s as UIElement;

            if (sender == null)
            {
                return;
            }

            var transform = sender.RenderTransform as RotateTransform;
            if (transform == null)
            {
                return;
            }

            transform.Angle += 45;
        }

        public static void Detach(UIElement element)
        {
            SetInitialAngle(element, 400.0);
            element.MouseLeftButtonDown -= RotateElement;
        }
    }
}
