﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace CustomControlsLibrary
{
    [TemplatePart(Name = "IncrementPart", Type = typeof(UIElement))]
    [TemplatePart(Name = "DecrementPart", Type = typeof(UIElement))]
    [TemplateVisualState(GroupName = "Common", Name = "Normal")]
    [TemplateVisualState(GroupName = "Common", Name = "Disabled")]
    [TemplateVisualState(GroupName = "Threshold", Name = "High")]
    [TemplateVisualState(GroupName = "Threshold", Name = "Equal")]
    [TemplateVisualState(GroupName = "Threshold", Name = "Low")]
    public class ThresholdControl : Control
    {
        public double Value
        {
            get
            {
                return (double)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        public static readonly DependencyProperty ValueProperty 
            = DependencyProperty.Register(
                "Value",
                typeof(double),
                typeof(ThresholdControl),
                new PropertyMetadata(10.0, OnValueChanged));

        public double Threshold
        {
            get
            {
                return (double)GetValue(ThresholdProperty);
            }
            set
            {
                SetValue(ThresholdProperty, value);
            }
        }

        public static readonly DependencyProperty ThresholdProperty
            = DependencyProperty.Register(
                "Threshold",
                typeof(double),
                typeof(ThresholdControl),
                new PropertyMetadata(0.0, OnValueChanged));

        private static void OnValueChanged(
            object s, 
            DependencyPropertyChangedEventArgs e)
        {
            var sender = s as ThresholdControl;
            if (sender != null)
            {
                sender.GoToThresholdState(true);
            }
        }

        private void GoToThresholdState(bool useTransitions)
        {
            if (Template == null)
            {
                return;
            }

            if (Value > Threshold)
            {
                VisualStateManager.GoToState(this, "High", useTransitions);
            }
            else
            {
                if (Value < Threshold)
                {
                    VisualStateManager.GoToState(this, "Low", useTransitions);
                }
                else
                {
                    VisualStateManager.GoToState(this, "Equal", useTransitions);
                }
            }
        }

        public ThresholdControl()
        {
            this.DefaultStyleKey = typeof(ThresholdControl);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            var incremementPart 
                = GetTemplateChild("IncrementPart") as UIElement;

            if (incremementPart != null)
            {
                incremementPart.MouseLeftButtonDown 
                    += IncremementPartMouseLeftButtonDown;
            }

            var decremementPart = GetTemplateChild("DecrementPart") as UIElement;

            if (decremementPart != null)
            {
                decremementPart.MouseLeftButtonDown 
                    += DecremementPartMouseLeftButtonDown;
            }

            GoToThresholdState(false);
        }

        void IncremementPartMouseLeftButtonDown(
            object sender, 
            MouseButtonEventArgs e)
        {
            Value++;
        }

        void DecremementPartMouseLeftButtonDown(
            object sender, 
            MouseButtonEventArgs e)
        {
            Value--;
        }
    }
}
