﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.IO;

namespace Customers
{
    public class CustomerAccess
    {
        public const string FileName = @"d:\temp\Customers.xml";
        //public const string FileName = @"e:\web\galasoftlbc\htdocs\labs\Customers\Customers.xml";

        public const string UrlBase = "http://localhost:55271/";
        //public const string UrlBase = "http://www.galasoft.ch/labs/Customers/";

        public IEnumerable<Customer> Get()
        {
            var document = XDocument.Load(FileName);

            var customers = from customer
                            in document.Descendants("Customer")
                            select new Customer
                            {
                                AccountNumber = Int32.Parse(customer.Attribute("AccountNumber").Value),
                                AddressCity = customer.Attribute("AddressCity").Value,
                                AddressCountry = customer.Attribute("AddressCountry").Value,
                                AddressStreet = customer.Attribute("AddressStreet").Value,
                                FirstName = customer.Attribute("FirstName").Value,
                                LastName = customer.Attribute("LastName").Value,
                                Gender = (CustomerGender)Enum.Parse(typeof(CustomerGender),
                                  customer.Attribute("Gender").Value),
                                PictureUri = new Uri(Path.Combine(UrlBase,
                                 customer.Attribute("Picture").Value).Replace("\\", "/"), UriKind.Absolute),
                            };

            return customers;
        }

        public IEnumerable<Customer> Get(string lastName)
        {
            return Get().Where(c => (c.LastName == lastName));
        }

        public Customer Get(int acountNumber)
        {
            try
            {
                return Get().First(c => (c.AccountNumber == acountNumber));
            }
            catch (Exception)
            {
                return null;
            }
        }

        public bool Save(IEnumerable<Customer> customers)
        {
            var document = XDocument.Load(FileName);

            try
            {
                foreach (var customer in customers)
                {
                    var element
                      = document.Descendants("Customer").First(c =>
                        Int32.Parse(c.Attribute("AccountNumber").Value) == customer.AccountNumber);

                    if (element == null)
                    {
                        continue;
                    }

                    element.SetAttributeValue("AddressCity", customer.AddressCity);
                    element.SetAttributeValue("AddressCountry", customer.AddressCountry);
                    element.SetAttributeValue("AddressStreet", customer.AddressStreet);
                    element.SetAttributeValue("FirstName", customer.FirstName);
                    element.SetAttributeValue("LastName", customer.LastName);
                }

                document.Save(FileName);
                return true;
            }
            catch (InvalidOperationException)
            {
                return false;
            }
        }
    }
}
