﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using Customers.Properties;

namespace Customers
{
    public class CustomerAccess
    {
        public const string XmlFileName = "Customers.xml";
        public const string PicturesFolderName = "pics";

        public IEnumerable<Customer> Get()
        {
            var fileName = Path.Combine(Settings.Default.PathBase, XmlFileName);
            var urlBase = Settings.Default.UrlBase;

            var document = XDocument.Load(fileName);

            var customers = from customer
                            in document.Descendants("Customer")
                            select new Customer
                            {
                                AccountNumber = Int32.Parse(customer.Attribute("AccountNumber").Value),
                                FirstName = customer.Attribute("FirstName").Value,
                                LastName = customer.Attribute("LastName").Value,
                                Gender = (CustomerGender) Enum.Parse(
                                    typeof (CustomerGender),
                                    customer.Attribute("Gender").Value),
                                PictureUri = new Uri(
                                Path.Combine(
                                    urlBase,
                                    customer.Attribute("Picture").Value).Replace("\\", "/"), UriKind.Absolute),
                            };

            return customers;
        }

        public IEnumerable<Customer> Get(string lastName)
        {
            return Get().Where(
                c => (c.LastName == lastName));
        }

        public Customer Get(int acountNumber)
        {
            try
            {
                return Get().First(
                    c => (c.AccountNumber == acountNumber));
            }
            catch (Exception)
            {
                return null;
            }
        }

        public bool Save(Customer customer)
        {
            var fileName = Path.Combine(Settings.Default.PathBase, XmlFileName);

            var document = XDocument.Load(fileName);

            try
            {
                var elements = document.Descendants("Customer").Where(
                    c => Int32.Parse(c.Attribute("AccountNumber").Value) == customer.AccountNumber);

                if (elements.Count() == 0)
                {
                    // This is a new customer
                    var element = new XElement("Customer");
                    element.SetAttributeValue("FirstName", customer.FirstName);
                    element.SetAttributeValue("LastName", customer.LastName);
                    element.SetAttributeValue("AccountNumber", customer.AccountNumber);
                    element.SetAttributeValue("Gender", customer.Gender);

                    var pictureDirectory = new DirectoryInfo(Path.Combine(Settings.Default.PathBase, PicturesFolderName));
                    var pictureFile =
                        new FileInfo(
                            Path.Combine(pictureDirectory.FullName, customer.AccountNumber + ".png"));

                    element.SetAttributeValue(
                        "Picture",
                        !pictureFile.Exists ? "pics/unknown.png" : string.Format("pics/{0}.png", customer.AccountNumber));

                    var customers = document.FirstNode as XElement;
                    customers.Add(element);
                }
                else
                {
                    var element = elements.First();
                    element.SetAttributeValue("FirstName", customer.FirstName);
                    element.SetAttributeValue("LastName", customer.LastName);
                    element.SetAttributeValue("Gender", customer.Gender);

                    var uri = customer.PictureUri;
                    var path = uri.AbsolutePath.Substring(1);
                    element.SetAttributeValue("Picture", path);
                }

                document.Save(fileName);
                return true;
            }
            catch (InvalidOperationException)
            {
                return false;
            }
        }
    }
}