﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using System.Text;

namespace Customers
{
    [ServiceContract(Namespace = "http://www.galasoft.ch")]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class CustomerService
    {
        private static Uri NotificationPushUri = null;

        [OperationContract]
        public IEnumerable<Customer> GetCustomers(string lastName)
        {
            var factory = new CustomerAccess();
            var customersFound = factory.Get(lastName);

            // Simulate a busy server by sleeping 5 seconds
            System.Threading.Thread.Sleep(5000);

            return customersFound;
        }

        [OperationContract]
        public Customer GetCustomerByAccountNumber(int accountNumber)
        {
            var factory = new CustomerAccess();
            var customerFound = factory.Get(accountNumber);
            return customerFound;
        }

        [OperationContract]
        public IEnumerable<Customer> GetAllCustomers()
        {
            var factory = new CustomerAccess();
            var customersFound = factory.Get();
            return customersFound;
        }

        [OperationContract]
        public bool SaveCustomer(Customer editedCustomer)
        {
            var factory = new CustomerAccess();
            var result = factory.Save(editedCustomer);
            return result;
        }

        [OperationContract]
        [WebGet(ResponseFormat = WebMessageFormat.Json,
            UriTemplate = "Save/{accountNumber}/{newName}")]
        public bool SaveCustomerLastName(string accountNumber, string newName)
        {
            Customer customer = null;

            try
            {
                var account = Int32.Parse(accountNumber);
                customer = GetCustomerByAccountNumber(account);
            }
            catch (FormatException)
            {
            }

            if (customer != null)
            {
                customer.LastName = newName;
                return SaveCustomer(customer);
            }
            return false;
        }
    }
}