﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using Customers.Properties;

namespace Customers
{
    public class SavePictureService : IHttpHandler
    {
        public const string CustomerAccountKey = "account";

        public void ProcessRequest(HttpContext context)
        {
            context.Response.ContentType = "text/plain";
            var request = context.Request;

            if (request.QueryString == null
                || string.IsNullOrEmpty(request.QueryString[CustomerAccountKey]))
            {
                context.Response.Write("Missing key 'account' in query string");
                return;
            }
            
            var bytes = new byte[context.Request.InputStream.Length];
            request.InputStream.Read(bytes, 0, bytes.Length);

            var outputPath = Path.Combine(Settings.Default.PathBase, CustomerAccess.PicturesFolderName);
            if (!Directory.Exists(outputPath))
            {
                try
                {
                    Directory.CreateDirectory(outputPath);
                }
                catch (Exception ex)
                {
                    context.Response.Write(ex.Message);
                    return;
                }
            }

            var accountNumber = Int32.Parse(request.QueryString[CustomerAccountKey]);
            var outputFilePath = Path.Combine(
                outputPath,
                string.Format("{0}.png", accountNumber));

            File.WriteAllBytes(outputFilePath, bytes);

            // Update customer file
            var customerService = new CustomerService();
            var customer = customerService.GetCustomerByAccountNumber(accountNumber);
            if (customer != null)
            {
                customer.PictureUri = new Uri(string.Format("pics/{0}.png", accountNumber), UriKind.Relative);
                customerService.SaveCustomer(customer);
            }

            request.InputStream.Close();
            context.Response.Write("Success");
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }
    }
}