﻿using System.ComponentModel.DataAnnotations;
using System.Windows;
using SampleMvvm.MvvmHelpers;
using DataApplicationSample.CustomerWcfService;
using System.ComponentModel;

namespace DataApplicationSample.ViewModel
{
    public class CustomerViewModel : ViewModelBase /*, IDataErrorInfo*/
    {
        public Customer Model
        {
            get;
            private set;
        }

        /// <summary>
        /// The <see cref="DirtyVisibility" /> property's name.
        /// </summary>
        public const string DirtyVisibilityPropertyName = "DirtyVisibility";

        private Visibility _dirty = Visibility.Collapsed;

        /// <summary>
        /// Gets the DirtyVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility DirtyVisibility
        {
            get
            {
                return _dirty;
            }

            set
            {
                if (_dirty == value)
                {
                    return;
                }

                _dirty = value;
                RaisePropertyChanged(DirtyVisibilityPropertyName);
            }
        }

        public CustomerViewModel(Customer model)
        {
            Model = model;
            model.PropertyChanged += (s, e) =>
            {
                DirtyVisibility = Visibility.Visible;

                if (e.PropertyName == LastNamePropertyName)
                {
                    RaisePropertyChanged(LastNamePropertyName);
                    return;
                }
            };
        }

        public string LastNamePropertyName = "LastName";

        [Required(
            ErrorMessage="The last name must be entered")]
        [RegularExpression(
            "^[A-Z]+[a-zA-Z]*$", 
            ErrorMessage="Only letters allowed, must start with a capital")]
        public string LastName
        {
            get { return Model.LastName; }
            set
            {
                if (Model.LastName == value)
                {
                    return;
                }

                Validate(value, LastNamePropertyName);
                Model.LastName = value;
            }
        }

        private void Validate(object value, string propertyName)
        {
            Validator.ValidateProperty(value,
                new ValidationContext(this, null, null)
                {
                    MemberName = propertyName
                });
        }

        //public string Error
        //{
        //    get { return null; }
        //}

        //public string this[string columnName]
        //{
        //    get
        //    {
        //        if (columnName == LastNamePropertyName)
        //        {
        //            if (string.IsNullOrEmpty(LastName))
        //            {
        //                return "The Last Name cannot be empty";
        //            }

        //            var firstChar = LastName[0];

        //            if (firstChar < 'A'
        //                || firstChar > 'Z')
        //            {
        //                return "The first character of the last name must be a capital letter";
        //            }
        //        }

        //        return null;
        //    }
        //}
    }
}
