﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using SampleMvvm.MvvmHelpers;
using DataApplicationSample.CustomerWcfService;
using DataApplicationSample.Model;

namespace DataApplicationSample.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        public ObservableCollection<CustomerViewModel> Customers
        {
            get;
            private set;
        }

        /// <summary>
        /// The <see cref="ErrorMessage" /> property's name.
        /// </summary>
        public const string ErrorMessagePropertyName = "ErrorMessage";

        private string _errorMessage = string.Empty;

        /// <summary>
        /// Gets the ErrorMessage property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public string ErrorMessage
        {
            get
            {
                return _errorMessage;
            }

            set
            {
                if (_errorMessage == value)
                {
                    return;
                }

                _errorMessage = value;
                RaisePropertyChanged(ErrorMessagePropertyName);
            }
        }

        private ICustomerServiceProxy _service;

        public RelayCommand SaveCommand
        {
            get;
            private set;
        }

        public MainViewModel(ICustomerServiceProxy service)
        {
            Customers = new ObservableCollection<CustomerViewModel>();
            _service = service;
            _service.GetCustomers(HandleResult);

            SaveCommand = new RelayCommand(SaveCustomers);
        }

        private void HandleResult(IEnumerable<Customer> result, Exception ex)
        {
            Customers.Clear();

            if (ex != null)
            {
                ErrorMessage = ex.Message;
                return;
            }

            if (result == null)
            {
                return;
            }

            foreach (var customer in result)
            {
                var vm = new CustomerViewModel(customer);
                Customers.Add(vm);
            }
        }

        private void SaveCustomers()
        {
            var collection = new List<Customer>();

            foreach (var customer in Customers)
            {
                if (customer.DirtyVisibility == Visibility.Visible)
                {
                    collection.Add(customer.Model);
                }
            }

            if (collection.Count > 0)
            {
                _service.SaveCustomers(collection, AfterSave);
            }
        }

        private void AfterSave(bool success)
        {
            if (success)
            {
                foreach (var customer in Customers)
                {
                    customer.DirtyVisibility = Visibility.Collapsed;
                }
            }
        }
    }
}