﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace DownloadUpload.Helpers
{
    public class DownloadUploadHelper
    {
        private const string ServerPath = "http://localhost:8975/";
        private const string FileName = "MyFile.txt";
        private const string ImageName = "MyImage.jpg";
        private const string SendStringService = "UploadString.ashx";
        private const string SendImageService = "UploadImage.ashx";

        public static void GetString(Action<string, Exception> callback)
        {
            var client = new WebClient();
            client.DownloadStringCompleted += GetStringCompleted;
            client.DownloadStringAsync(
                new Uri(ServerPath + FileName), 
                callback);
        }

        private static void GetStringCompleted(
            object sender, 
            DownloadStringCompletedEventArgs e)
        {
            var callback = e.UserState as Action<string, Exception>;
            if (callback == null
                || e.Cancelled)
            {
                return;
            }

            if (e.Error != null)
            {
                callback(null, e.Error);
                return;
            }

            // Everything OK
            callback(e.Result, null);
        }

        public static void SendString(string content, Action<string, Exception> callback)
        {
            var client = new WebClient();
            client.UploadStringCompleted += SendStringCompleted;
            client.UploadStringAsync(
                new Uri(ServerPath + SendStringService), 
                //"PUT", // Only works with new HTTP stack
                "POST", // Works always
                content, 
                callback);
        }

        private static void SendStringCompleted(object sender, UploadStringCompletedEventArgs e)
        {
            var callback = e.UserState as Action<string, Exception>;
            if (callback == null
                || e.Cancelled)
            {
                return;
            }

            if (e.Error != null)
            {
                callback(null, e.Error);
                return;
            }

            // Everything OK
            callback(e.Result, null);
        }

        private static Action<int> _progressCallback;

        public static void GetImage(Action<ImageSource, Exception> callback, Action<int> progressCallback)
        {
            var client = new WebClient();
            client.OpenReadCompleted += GetImageCompleted;
            client.OpenReadAsync(new Uri(ServerPath + ImageName), callback);
            if (progressCallback != null)
            {
                _progressCallback = progressCallback;
                client.DownloadProgressChanged += GetImageProgressChanged;
            }
        }

        private static void GetImageProgressChanged(object sender, DownloadProgressChangedEventArgs e)
        {
            if (_progressCallback != null)
            {
                _progressCallback(e.ProgressPercentage);
            }
        }

        private static void GetImageCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            _progressCallback = null;

            var callback = e.UserState as Action<ImageSource, Exception>;
            if (callback == null
                || e.Cancelled)
            {
                callback(null, null);
                return;
            }

            if (e.Error != null)
            {
                callback(null, e.Error);
                return;
            }

            // Everything OK

            var image = new BitmapImage();
            image.SetSource(e.Result);
            callback(image, null);
        }

        public static void SendImage(
            FileInfo content, 
            Action<string, Exception> callback)
        {
            var client = new WebClient();
            client.OpenWriteCompleted += SendImageCompleted;

            var info = new SendImageInfo
            {
                Content = content,
                Callback = callback
            };

            client.OpenWriteAsync(
                new Uri(ServerPath + SendImageService),
                "POST", // Works always
                //"PUT", // Works only with new HTTP stack
                info);
        }

        private static void SendImageCompleted(
            object sender, 
            OpenWriteCompletedEventArgs e)
        {
            if (e.Cancelled)
            {
                return;
            }

            var info = (SendImageInfo)e.UserState;

            if (e.Error != null)
            {
                info.Callback(null, e.Error);
                return;
            }

            var inputStream = info.Content.OpenRead();
            var outputStream = e.Result;

            ReadWriteFile(inputStream, outputStream);

            // Everything OK
            info.Callback("File sent, no server response to show", null);
        }

        public static void ReadWriteFile(
            Stream inputStream, 
            Stream outputStream)
        {
            using (var writer = new BinaryWriter(outputStream))
            {
                using (var reader = new BinaryReader(inputStream))
                {
                    var stop = false;
                    while (!stop)
                    {
                        var bytes = reader.ReadBytes(1000);
                        writer.Write(bytes);

                        if (bytes.Length < 1000)
                        {
                            stop = true;
                        }
                    }
                }
            }
        }

        private struct SendImageInfo
        {
            public FileInfo Content;
            public Action<string, Exception> Callback;
        }
    }
}
