﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Threading;

namespace DownloadUpload.Helpers
{
    public class WebRequestHelper
    {
        private const string ServerPath = "http://localhost:8975/";
        private const string SendImageService = "UploadImage.ashx";
        private const string NewImageName = "MyImage{0}.jpg";

        public static void SendImage(
            FileInfo content, 
            Action<string, Exception> callback)
        {
            var request = (HttpWebRequest)WebRequest.Create(
                new Uri(ServerPath + SendImageService));

            request.Method = "POST"; // Works always
            //request.Method = "PUT";  // Only works with new HTTP stack

            var info = new SendImageInfo
            {
                Request = request,
                Callback = callback,
                Content = content,
                MainThreadContext = SynchronizationContext.Current
            };

            request.BeginGetRequestStream(SendImageRequestCallback, info);
        }

        private static void SendImageRequestCallback(IAsyncResult result)
        {
            var info = (SendImageInfo)result.AsyncState;
            var request = info.Request;

            var inputStream = info.Content.OpenRead();
            var outputStream = request.EndGetRequestStream(result);

            DownloadUploadHelper.ReadWriteFile(inputStream, outputStream);

            request.BeginGetResponse(ResponseCallback, info);
        }

        private static void ResponseCallback(IAsyncResult result)
        {
            var info = (SendImageInfo)result.AsyncState;

            try
            {
                var response = info.Request.EndGetResponse(result);
                info.Response = response;
            }
            catch (Exception ex)
            {
                info.Error = ex;
            }

            info.MainThreadContext.Post(MainThreadCallback, info);
        }

        private static void MainThreadCallback(object state)
        {
            var info = (SendImageInfo)state;
            if (info.Error != null)
            {
                info.Callback(null, info.Error);
                return;
            }

            var response = info.Response as HttpWebResponse;
            var stream = info.Response.GetResponseStream();

            using (var reader = new StreamReader(stream))
            {
                var serverMessage = reader.ReadToEnd();
                info.Callback(serverMessage, null);
            }
        }

        private struct SendImageInfo
        {
            public WebRequest Request;
            public Action<string, Exception> Callback;
            public FileInfo Content;
            public SynchronizationContext MainThreadContext;
            public WebResponse Response;
            public Exception Error;
        }
    }
}
