﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using DownloadUpload.Helpers;
using System.Net.Browser;

namespace DownloadUpload
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            // Uncomment to use the new HTTP stack

            bool httpResult = WebRequest.RegisterPrefix(
                "http://",
                WebRequestCreator.ClientHttp);

            //bool httpsResult = WebRequest.RegisterPrefix(
            //    "https://", 
            //    WebRequestCreator.ClientHttp);
        }

        private void SendFileClick(object sender, RoutedEventArgs e)
        {
            var dialog = new OpenFileDialog();
            dialog.Filter = "JPEG files|*.jpg";

            if (dialog.ShowDialog() == true)
            {
                Cache.Visibility = Visibility.Visible;
                DownloadUploadHelper.SendImage(
                    dialog.File,
                    SendFileCallback);
            }
        }

        private void SendFileCallback(string result, Exception error)
        {
            if (error != null)
            {
                var message = error.Message;
                var webError = error as WebException;
                if (webError != null)
                {
                    var response = webError.Response as HttpWebResponse;
                    if (response != null)
                    {
                        message += " (code " + response.StatusCode + ")";
                    }
                }

                MessageBox.Show(message);
            }
            else
            {
                MessageBox.Show(result);
            }

            Cache.Visibility = Visibility.Collapsed;
        }

        private void SendStringClick(object sender, RoutedEventArgs e)
        {
            Cache.Visibility = Visibility.Visible;

            DownloadUploadHelper.SendString(
                MyText.Text,
                (result, error) =>
                {
                    if (error != null)
                    {
                        HandleError(error);
                    }
                    else
                    {
                        if (result != null)
                        {
                            MessageBox.Show(result);
                        }
                    }

                    Cache.Visibility = Visibility.Collapsed;
                });
        }

        private void HandleError(Exception error)
        {
            var message = error.GetType().FullName
              + " " + error.Message;
            var webError = error as WebException;
            if (webError != null)
            {
                var response = webError.Response as HttpWebResponse;
                if (response != null)
                {
                    message += " (code " + response.StatusCode + ")";
                }
            }

            MessageBox.Show(message);
        }

        private void GetImageClick(object sender, RoutedEventArgs e)
        {
            Cache.Visibility = Visibility.Visible;
            Status.Text = string.Empty;
            MyImage.Source = null;

            DownloadUploadHelper.GetImage(
                (result, error) =>
                {
                    if (error != null)
                    {
                        HandleError(error);
                    }
                    else
                    {
                        MyImage.Source = result;
                    }

                    Cache.Visibility = Visibility.Collapsed;
                    Status.Text = string.Empty;
                },
                i => Status.Text = i.ToString() + " %");
        }

        private void GetStringClick(object sender, RoutedEventArgs e)
        {
            Cache.Visibility = Visibility.Visible;

            DownloadUploadHelper.GetString((result, error) =>
            {
                if (error != null)
                {
                    HandleError(error);
                }
                else
                {
                    MessageBox.Show(result);
                }

                Cache.Visibility = Visibility.Collapsed;
            });
        }

        private void ResetClick(object sender, RoutedEventArgs e)
        {
            MyText.Text = string.Empty;
            MyImage.Source = null;
        }

        private void SendFileRequestClick(object sender, RoutedEventArgs e)
        {
            var dialog = new OpenFileDialog();
            dialog.Filter = "JPEG files|*.jpg";

            if (dialog.ShowDialog() == true)
            {
                Cache.Visibility = Visibility.Visible;
                WebRequestHelper.SendImage(
                    dialog.File,
                    SendFileCallback);
            }
        }

        private void TestCookiesClick(object sender, RoutedEventArgs e)
        {
            CookieSample.SendCookie();
        }
    }
}
