﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using DuplexPollingSample.FileChangedService;

namespace DuplexPollingSample
{
    public partial class MainPage : UserControl
    {
        private FilesChangedServiceClient _client;
        public ObservableCollection<string> Files
        {
            get;
            private set;
        }

        public MainPage()
        {
            Files = new ObservableCollection<string>();
            InitializeComponent();

            var source = App.Current.Host.Source;
            var baseAddress = source.AbsoluteUri.Substring(
                0,
                source.AbsoluteUri.Length - source.AbsolutePath.Length);
            var servicePath = baseAddress + "/FilesChangedService.svc";
            var serviceAddress = new EndpointAddress(servicePath);
            var binding = new PollingDuplexHttpBinding();
            
            _client = new FilesChangedServiceClient(binding, serviceAddress);
            _client.FileAddedReceived += FileAdded;
            _client.FileDeletedReceived += FileDeleted;
            _client.FileRenamedReceived += FileRenamed;
            _client.StartObservingFilesAsync();
        }

        private void FileAdded(
            object sender,
            FileAddedReceivedEventArgs e)
        {
            if (!Files.Contains(e.fileName))
            {
                Files.Add(e.fileName);
                ShowNotification();
            }
        }

        private void FileRenamed(object sender, FileRenamedReceivedEventArgs e)
        {
            if (Files.Contains(e.oldFileName))
            {
                var index = Files.IndexOf(e.oldFileName);
                Files[index] = e.fileName;
                ShowNotification();
            }
        }

        private void FileDeleted(object sender, FileDeletedReceivedEventArgs e)
        {
            if (Files.Contains(e.fileName))
            {
                Files.Remove(e.fileName);
                ShowNotification();
            }
        }

        private void UnsubscribeButtonClick(object sender, RoutedEventArgs e)
        {
            _client.StopObservingFilesAsync();
            UnsubscribeButton.IsEnabled = false;
            ResubscribeButton.IsEnabled = true;
        }

        private void ResubscribeButtonClick(object sender, RoutedEventArgs e)
        {
            Files.Clear();
            _client.StartObservingFilesAsync();
            UnsubscribeButton.IsEnabled = true;
            ResubscribeButton.IsEnabled = false;
        }

        private bool _isNotificationOpen;

        private void ShowNotification()
        {
            if (_isNotificationOpen
                || !App.Current.IsRunningOutOfBrowser)
            {
                return;
            }

            var win = new NotificationWindow();
            var uc = new NotificationContent();
            win.Width = uc.Width;
            win.Height = uc.Height;
            win.Content = uc;
            win.Closed += (s, e) => _isNotificationOpen = false;
            uc.MouseLeftButtonDown += (s, e) =>
            {
                if (App.Current.MainWindow.WindowState == WindowState.Minimized)
                {
                    App.Current.MainWindow.WindowState = WindowState.Normal;
                }

                App.Current.MainWindow.Activate();
            };
            uc.CloseRequested += (s, e) => win.Close();
            _isNotificationOpen = true;
            win.Show(5000);
        }
    }
}
