﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Windows;
using System.Windows.Controls;

namespace EncryptionSample
{
    public partial class MainPage : UserControl
    {
        private const string PasswordSalt = "PasswordSalt";

        public MainPage()
        {
            InitializeComponent();
        }

        private void DecryptClick(object sender, RoutedEventArgs e)
        {
            var dialog = new OpenFileDialog
            {
                Filter = "Text files (*.txt)|*.txt"
            };

            if (dialog.ShowDialog() == true)
            {
                var deriveBytes = new Rfc2898DeriveBytes(
                    PasswordTextBox.Password,
                    Encoding.UTF8.GetBytes(PasswordSalt));

                using (var decrypt = new AesManaged())
                {
                    decrypt.Key = deriveBytes.GetBytes(128 / 8);

                    using (var inputStream = dialog.File.OpenRead())
                    {
                        decrypt.IV = ReadByteArray(inputStream);

                        try
                        {
                            using (var cs = new CryptoStream(inputStream, decrypt.CreateDecryptor(), CryptoStreamMode.Read))
                            {
                                using (var reader = new StreamReader(cs, Encoding.Unicode))
                                {
                                    OutputTextBox.Text = reader.ReadToEnd();
                                }
                            }
                        }
                        catch (Exception error)
                        {
                            OutputTextBox.Text = "Impossible to decrypt";
                        }
                    }
                }
            }
        }

        private void EncryptClick(object sender, RoutedEventArgs e)
        {
            var dialog = new SaveFileDialog
            {
                Filter = "Text files (*.txt)|*.txt"
            };

            if (dialog.ShowDialog() == true)
            {
                var deriveBytes = new Rfc2898DeriveBytes(
                    PasswordTextBox.Password, 
                    Encoding.UTF8.GetBytes(PasswordSalt));

                using (var encrypt = new AesManaged())
                {
                    encrypt.Key = deriveBytes.GetBytes(128 / 8);

                    using (var outputStream = dialog.OpenFile())
                    {
                        outputStream.Write(BitConverter.GetBytes(encrypt.IV.Length), 0, sizeof(int));
                        outputStream.Write(encrypt.IV, 0, encrypt.IV.Length);

                        using (var cs = new CryptoStream(outputStream, encrypt.CreateEncryptor(), CryptoStreamMode.Write))
                        {
                            var rawPlaintext = Encoding.Unicode.GetBytes(InputTextBox.Text);
                            cs.Write(rawPlaintext, 0, rawPlaintext.Length);
                            cs.FlushFinalBlock();
                        }
                    }
                }
            }
        }

        private static byte[] ReadByteArray(Stream s)
        {
            var rawLength = new byte[sizeof(int)];
            if (s.Read(rawLength, 0, rawLength.Length) != rawLength.Length)
            {
                throw new SystemException("Stream did not contain properly formatted byte array");
            }

            var buffer = new byte[BitConverter.ToInt32(rawLength, 0)];
            if (s.Read(buffer, 0, buffer.Length) != buffer.Length)
            {
                throw new SystemException("Did not read byte array properly");
            }

            return buffer;
        }
    }
}
