﻿using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Windows;
using System.Windows.Shapes;

namespace GetAnimationBaseValue
{
    public partial class MainPage : UserControl
    {
        public const string RectangleBaseValuePropertyName = "RectangleBaseValue";

        public double RectangleBaseValue
        {
            get
            {
                return (double)GetValue(RectangleBaseValueProperty);
            }
            set
            {
                SetValue(RectangleBaseValueProperty, value);
            }
        }

        public static readonly DependencyProperty RectangleBaseValueProperty 
            = DependencyProperty.Register(
            RectangleBaseValuePropertyName,
            typeof(double),
            typeof(MainPage),
            new PropertyMetadata(0.0));

        public const string RectangleLocalValuePropertyName = "RectangleLocalValue";

        public double RectangleLocalValue
        {
            get
            {
                return (double)GetValue(RectangleLocalValueProperty);
            }
            set
            {
                SetValue(RectangleLocalValueProperty, value);
            }
        }

        public static readonly DependencyProperty RectangleLocalValueProperty = DependencyProperty.Register(
            RectangleLocalValuePropertyName,
            typeof(double),
            typeof(MainPage),
            new PropertyMetadata(0.0));

        public MainPage()
        {
            InitializeComponent();

            ReadValues();

            var sbd = Resources["AnimateHeightStoryboard"] as Storyboard;
            sbd.Begin();
        }

        private double _rectangleHeight = 100.0;

        private void IncreaseLocalValueButtonClick(object sender, RoutedEventArgs e)
        {
            _rectangleHeight += 10.0;
            MyRectangle.Height = _rectangleHeight;

            ReadValues();
        }

        private void ReadValues()
        {
            RectangleBaseValue
                = (double)MyRectangle.GetAnimationBaseValue(
                HeightProperty);

            //var localValue
            //    = MyRectangle.ReadLocalValue(
            //    HeightProperty);
            //if (localValue == DependencyProperty.UnsetValue)
            //{
            //    RectangleLocalValue = -1;
            //}
            //else
            //{
            //    RectangleLocalValue
            //        = (double)MyRectangle.ReadLocalValue(
            //        HeightProperty);
            //}

            double? rectangleHeight = null;
            var localValue
                = MyRectangle.ReadLocalValue(
                HeightProperty);
            if (localValue != DependencyProperty.UnsetValue)
            {
                rectangleHeight
                    = (double)MyRectangle.ReadLocalValue(
                    HeightProperty);
            }
        }

        private void StartAnimationButtonClick(object sender, RoutedEventArgs e)
        {
            var sbd = Resources["AnimateHeightStoryboard"] as Storyboard;
            sbd.Begin();
        }

        private void StopAnimationButtonClick(object sender, RoutedEventArgs e)
        {
            var sbd = Resources["AnimateHeightStoryboard"] as Storyboard;
            sbd.Stop();
        }

        private void ClearValueButtonClick(object sender, RoutedEventArgs e)
        {
            MyRectangle.ClearValue(HeightProperty);
            ReadValues();
        }
    }
}
