﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using AudioRecorder.Audio;

namespace GreetingCardMaker.Helpers
{
    public class WavAudioSink : AudioSink
    {
        private Stream _stream;
        private AudioFormat _format;
        private bool _isRecording;

        protected override void OnCaptureStarted()
        {
        }

        protected override void OnCaptureStopped()
        {
        }

        protected override void OnFormatChange(AudioFormat audioFormat)
        {
            if (audioFormat.WaveFormat
                != WaveFormatType.Pcm)
            {
                throw new ArgumentException(
                    "Only PCM is supported",
                    "audioFormat");
            }

            _format = audioFormat;
        }

        protected override void OnSamples(long sampleTimeInHundredNanoseconds, long sampleDurationInHundredNanoseconds, byte[] sampleData)
        {
            if (_stream == null)
            {
                _isRecording = true;
                _stream = new MemoryStream();
            }

            if (_isRecording)
            {
                _stream.Write(sampleData, 0, sampleData.Length);
            }
        }

        public void Save(Stream outputStream)
        {
            _isRecording = false;
            CaptureSource = null;

            WavManager.SavePcmToWav(_stream, outputStream, _format);
            _stream = null;
        }
    }
}
