﻿using System.ComponentModel;
using System.Windows;

namespace ImplementingINotifyPropertyChanged.Data
{
public class Customer : INotifyPropertyChanged
{
    public event PropertyChangedEventHandler PropertyChanged;

    // Name Property

    public const string NamePropertyName 
        = "Name";

    private string _name = string.Empty;

    public string Name
    {
        get
        {
            return _name;
        }

        set
        {
            if (_name == value)
            {
                return;
            }

            _name = value;
            RaisePropertyChanged(NamePropertyName);
            NumberOfChanges++;
        }
    }

    // ContractNumber Dependency Property

    public const string ContractNumberPropertyName 
        = "ContractNumber";

    private int _contractNumber = 0;

    public int ContractNumber
    {
        get
        {
            return _contractNumber;
        }

        set
        {
            if (_contractNumber == value)
            {
                return;
            }

            _contractNumber = value;
            RaisePropertyChanged(ContractNumberPropertyName);
        }
    }

    public const string NumberOfChangesPropertyName = "NumberOfChanges";

    private int _numberOfChanges;

    public int NumberOfChanges
    {
        get
        {
            return _numberOfChanges;
        }

        set
        {
            if (_numberOfChanges == value)
            {
                return;
            }

            _numberOfChanges = value;
            RaisePropertyChanged(NumberOfChangesPropertyName);
        }
    }
    
    public Customer(string name, int contractNumber)
    {
        Name = name;
        ContractNumber = contractNumber;
    }

    private void RaisePropertyChanged(string propertyName)
    {
        if (PropertyChanged != null)
        {
            PropertyChanged(
                this, 
                new PropertyChangedEventArgs(propertyName));
        }
    }
}
}
