﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using MefHierarchy.ViewModel;

namespace MefHierarchy.Model
{
    public abstract class DataService : IDataService
    {
        public abstract string ServiceAddressUri
        {
            get;
        }

        public void GetItems(Action<IEnumerable<Item>> callback)
        {
            var client = new WebClient();
            client.DownloadStringCompleted += ClientDownloadStringCompleted;
            client.DownloadStringAsync(new Uri(ServiceAddressUri), callback);
        }

        private void ClientDownloadStringCompleted(
            object sender,
            DownloadStringCompletedEventArgs e)
        {
            var callback = e.UserState as Action<IEnumerable<Item>>;
            callback(ParseResult(e.Result));
        }

        private IEnumerable<Item> ParseResult(string serial)
        {
            var result = new List<Item>();

            // Parse the result and return items
            // The text file is simply a series of words, one word on each line.
            var items = serial.Split(
                new string[]
                {
                    Environment.NewLine
                },
                StringSplitOptions.RemoveEmptyEntries);

            foreach (var item in items)
            {
                result.Add(
                    new Item
                    {
                        Message = item
                    });
            }

            return result.AsEnumerable<Item>();
        }
    }
}
