﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.ComponentModel;
using MefHierarchy.Model;

namespace MefHierarchy.ViewModel
{
    [Export(typeof(ViewModelWithMef))]
    public class ViewModelWithMef : INotifyPropertyChanged
    {
        [ImportMany]
        public Lazy<IDataService, IDataServiceInfo>[] Services
        {
            get;
            set;
        }

        public const string ItemsPropertyName = "Items";
        private ObservableCollection<Item> _items;
        public ObservableCollection<Item> Items
        {
            get
            {
                return _items;
            }

            set
            {
                if (_items == value)
                {
                    return;
                }

                _items = value;
                RaisePropertyChanged(ItemsPropertyName);
            }
        }

        private IDataService CurrentService
        {
            get
            {
                IDataService currentService = null;

                foreach (var service in Services)
                {
                    if (DateTime.Now.Hour < 12)
                    {
                        if (service.Metadata.IsForTheMorning)
                        {
                            currentService = service.Value;
                            break;
                        }
                    }
                    else
                    {
                        if (!service.Metadata.IsForTheMorning)
                        {
                            currentService = service.Value;
                            break;
                        }
                    }
                }

                return currentService;
            }
        }

        public void GetItems()
        {
            CurrentService.GetItems(items =>
            {
                Items = new ObservableCollection<Item>(items);
            });
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
