﻿using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using GalaSoft.MvvmLight.Messaging;
using MefPlugins.BingImageSearch.BingSearchService;
using MefPlugins.Contracts;

namespace MefPlugins.BingImageSearch
{
    [Export(typeof(ISearchEngine))]
    public partial class BingImageSearch : UserControl, ISearchEngine
    {
        public const string MyAppId = "03B362A2B0F8A91CE20AE30296B946C5A2938772";

        private readonly BingPortTypeClient _service;

        public const string ResultsPropertyName = "Results";
        public ObservableCollection<ImageResult> Results
        {
            get
            {
                return (ObservableCollection<ImageResult>)GetValue(ResultsProperty);
            }
            set
            {
                SetValue(ResultsProperty, value);
            }
        }

        public static readonly DependencyProperty ResultsProperty = DependencyProperty.Register(
            ResultsPropertyName,
            typeof(ObservableCollection<ImageResult>),
            typeof(BingImageSearch),
            new PropertyMetadata(null));

        public BingImageSearch()
        {
            _service = new BingPortTypeClient();
            _service.SearchCompleted += HandleResponse;

            InitializeComponent();
        }

        public string SearchType
        {
            get
            {
                return "Image";
            }
        }

        public void Search(string query)
        {
            var request = new SearchRequest
            {
                AppId = MyAppId,
                Query = query,
                Sources = new[]
                {
                    SourceType.Image
                },
                Image = new ImageRequest
                {
                    Count = 100
                }
            };

            _service.SearchAsync(request);
        }

        private void HandleResponse(object sender, SearchCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message, "Error", MessageBoxButton.OK);
                return;
            }

            if (e.Result == null
                || e.Result.Image == null
                || e.Result.Image.Results == null)
            {
                MessageBox.Show("Unknown error");
                return;
            }

            if (e.Result.Image.Results.Count() == 0)
            {
                MessageBox.Show("No results");
                return;
            }

            Results = new ObservableCollection<ImageResult>(e.Result.Image.Results.AsEnumerable());

            Messenger.Default.Send(
                Results.Count,
                Notifications.NumberOfMessages);
        }

        public UIElement SearchView
        {
            get
            {
                return this;
            }
        }
    }
}
