﻿using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using GalaSoft.MvvmLight.Messaging;
using MefPlugins.BingTextSearch.BingSearchService;
using MefPlugins.Contracts;
using WebRequest = MefPlugins.BingTextSearch.BingSearchService.WebRequest;

namespace MefPlugins.BingTextSearch
{
    [Export(typeof(ISearchEngine))]
    public partial class BingTextSearch : UserControl, ISearchEngine
    {
        public const string MyAppId = "03B362A2B0F8A91CE20AE30296B946C5A2938772";

        private readonly BingPortTypeClient _service;

        public const string ResultsPropertyName = "Results";
        public ObservableCollection<WebResult> Results
        {
            get
            {
                return (ObservableCollection<WebResult>)GetValue(ResultsProperty);
            }
            set
            {
                SetValue(ResultsProperty, value);
            }
        }

        public static readonly DependencyProperty ResultsProperty = DependencyProperty.Register(
            ResultsPropertyName,
            typeof(ObservableCollection<WebResult>),
            typeof(BingTextSearch),
            new PropertyMetadata(null));

        public BingTextSearch()
        {
            _service = new BingPortTypeClient();
            _service.SearchCompleted += HandleResponse;

            InitializeComponent();
        }

        public string SearchType
        {
            get
            {
                return "Web";
            }
        }

        public UIElement SearchView
        {
            get
            {
                return this;
            }
        }

        public void Search(string query)
        {
            var request = new SearchRequest
            {
                AppId = MyAppId,
                Query = query,
                Sources = new[]
                {
                    SourceType.Web
                },
                Web = new WebRequest
                {
                    Count = 100
                }
            };

            _service.SearchAsync(request);
        }

        private void HandleResponse(object sender, SearchCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message, "Error", MessageBoxButton.OK);
                return;
            }

            if (e.Result == null
                || e.Result.Web == null)
            {
                MessageBox.Show("Unknown error");
				return;
            }

            if (e.Result.Web.Results == null
                || e.Result.Web.Results.Count() == 0)
            {
                MessageBox.Show("No results");
                return;
            }

            Results = new ObservableCollection<WebResult>(e.Result.Web.Results.AsEnumerable());

            Messenger.Default.Send(
                Results.Count,
                Notifications.NumberOfMessages);
        }
    }
}
