﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using MefPlugins.Contracts;

namespace MefPlugins
{
    public partial class MainPage : UserControl
    {
        [ImportMany(AllowRecomposition = true)]
        public Lazy<ISearchEngine>[] Engines
        {
            get;
            set;
        }

        private static AggregateCatalog _aggregateCatalog;
        private CompositionContainer _container;
        private string _baseAddress;
        private Dictionary<string, DeploymentCatalog> _catalogs
            = new Dictionary<string, DeploymentCatalog>();

        public MainPage()
        {
            InitializeComponent();

            _aggregateCatalog = new AggregateCatalog();
            _aggregateCatalog.Catalogs.Add(
                new AssemblyCatalog(
                    typeof(BingTextSearch.BingTextSearch).Assembly));
            _container = new CompositionContainer(_aggregateCatalog);

            var xapUri = App.Current.Host.Source;
            _baseAddress = xapUri.AbsoluteUri.Substring(
                0, xapUri.AbsoluteUri.IndexOf(xapUri.AbsolutePath))
                + "/";

            _container.ComposeParts(this);
            Compose();
            LoadPlugins();
        }

        private void LoadPlugins()
        {
            var serviceAddress = _baseAddress
                + "PluginsService.ashx?"
                + DateTime.Now.Ticks;

            var client = new WebClient();
            client.DownloadStringCompleted += client_DownloadStringCompleted;
            client.DownloadStringAsync(new Uri(serviceAddress));
        }

        void client_DownloadStringCompleted(
            object sender,
            DownloadStringCompletedEventArgs e)
        {
            var plugins = e.Result.Split(
                new string[] { Environment.NewLine },
                StringSplitOptions.RemoveEmptyEntries);
            foreach (var plugin in plugins)
            {
                AddXap(_baseAddress + plugin);
            }
        }

        private void AddXap(string uri)
        {
            DeploymentCatalog catalog;
            if (!_catalogs.TryGetValue(uri, out catalog))
            {
                catalog = new DeploymentCatalog(
                    new Uri(uri, UriKind.Absolute));
                _aggregateCatalog.Catalogs.Add(catalog);
                catalog.DownloadCompleted += (s, e) =>
                {
                    if (e.Error == null
                        && !e.Cancelled)
                    {
                        Compose() ;
                    }
                } ;
                catalog.DownloadAsync();

                _catalogs[uri] = catalog ;
            }
            _aggregateCatalog.Catalogs.Add(catalog) ;
        }

        private void Compose()
        {
            foreach (var engine in Engines)
            {
                var found = false;
                foreach (TabItem tab in MyTabControl.Items)
                {
                    if (tab.Header.ToString()
                        == engine.Value.SearchType)
                    {
                        found = true;
                        break;
                    }
                }

                if (found)
                {
                    continue;
                }

                var newTab = new TabItem
                {
                    Header = engine.Value.SearchType,
                    Content = engine.Value.SearchView
                };

                MyTabControl.Items.Add(newTab);
            }
        }

        private void SearchClick(object sender, RoutedEventArgs e)
        {
            foreach (var engine in Engines)
            {
                engine.Value.Search(QueryTextBox.Text);
            }
        }

        private void RefreshClick(object sender, RoutedEventArgs e)
        {
            LoadPlugins();
        }
    }
}
