﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ServiceModel.DomainServices.Client;
using MyNorthwind.Web;
using System.Windows.Printing;

namespace MyNorthwind.Silverlight
{
    public partial class PrintWindow : ChildWindow
    {
        private OperationBase _operation;
        private bool _cancel;
        private int _printedItems;
        private IEnumerable<Order> _ordersToPrint;

        public PrintWindow(string cityStart)
        {
            InitializeComponent();
            OKButton.IsEnabled = false;

            if (string.IsNullOrEmpty(cityStart))
            {
                MessageTextBlock.Text = string.Format(
                    "Loading all orders",
                    cityStart);
            }
            else
            {
                MessageTextBlock.Text = string.Format(
                    "Loading orders for {0}",
                    cityStart);
            }

            LoadOrders(cityStart);
        }

        private void LoadOrders(string cityStart)
        {
            var service = new MyNorthwindDomainContext();
            _operation = service.Load(
                service.GetOrdersByCityQuery(cityStart),
                LoadCompleted,
                null);
        }

        private void LoadCompleted(LoadOperation<Order> op)
        {
            MessageTextBlock.Text = string.Format(
                "{0} orders loaded, ready to print",
                op.Entities.Count());

            _ordersToPrint = op.Entities;
            OKButton.IsEnabled = true;
        }

        private void OKButton_Click(object sender, RoutedEventArgs e)
        {
            var printDoc = new PrintDocument();
            printDoc.BeginPrint += BeginPrint;
            printDoc.PrintPage += PrintPage;
            printDoc.EndPrint += EndPrint;
            printDoc.Print("OrdersReport");
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            _cancel = true;

            if (_operation != null
                && _operation.CanCancel)
            {
                _operation.Cancel();
            }

            DialogResult = false;
        }

        private void BeginPrint(object sender, BeginPrintEventArgs e)
        {
            _printedItems = 0;
        }

        private void PrintPage(object sender, PrintPageEventArgs e)
        {
            var printedReport = new PrintedReport();
            e.PageVisual = printedReport;

            printedReport.PreparePage(
                _ordersToPrint,
                _printedItems,
                e.PrintableArea);

            _printedItems += printedReport.NumberOfOrders;
            e.HasMorePages = _printedItems < _ordersToPrint.Count();
        }

        private void EndPrint(object sender, EndPrintEventArgs e)
        {
            DialogResult = true;
        }
    }
}

