﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Interactivity;

namespace MyOwnBehaviors
{
	public class TextBoxListenerBehavior : Behavior<TextBox>
	{
		public TextBoxListenerBehavior()
		{
		}

        public const string MessagePropertyName = "Message";

        public string Message
        {
            get
            {
                return (string)GetValue(MessageProperty);
            }
            set
            {
                SetValue(MessageProperty, value);
            }
        }

        public static readonly DependencyProperty MessageProperty 
            = DependencyProperty.Register(
            MessagePropertyName,
            typeof(string),
            typeof(TextBoxListenerBehavior),
            new PropertyMetadata("-empty-"));

		protected override void OnAttached()
		{
			base.OnAttached();
            AssociatedObject.KeyUp += TextBoxKeyUp;
		}

        protected override void OnDetaching()
        {
            AssociatedObject.KeyUp -= TextBoxKeyUp;
            base.OnDetaching();
        }

        private string _textEntered = string.Empty;
        private const string ReferenceText = "hello";

        private void TextBoxKeyUp(object sender, KeyEventArgs e)
        {
            _textEntered += e.Key.ToString();

            if (ReferenceText.Equals(
                _textEntered, 
                StringComparison.InvariantCultureIgnoreCase))
            {
                MessageBox.Show(
                    string.Format("It is now {0}; your message is {1}",
                    DateTime.Now,
                    Message));
                _textEntered = string.Empty;
                return;
            }

            if (!ReferenceText.ToLower().StartsWith(
                _textEntered.ToLower()))
            {
                // Reset
                _textEntered = string.Empty;
            }
        }
	}
}