﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using GalaSoft.MvvmLight.Command;

namespace RightClickEvent.ViewModel
{
    public class MainViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public ObservableCollection<Customer> Customers
        {
            get;
            private set;
        }

        public RelayCommand<Customer> DeleteCustomerCommand
        {
            get;
            private set;
        }

        public MainViewModel()
        {
            // Building dummy data

            Customers = new ObservableCollection<Customer>();
            var random = new Random();

            for (var index = 0; index < 30; index++)
            {
                var customer = new Customer
                {
                    Name = "Dummy name " + index,
                    Account = random.Next(1000, 2000)
                };

                Customers.Add(customer);
            }

            DeleteCustomerCommand = new RelayCommand<Customer>(DeleteCustomer);
        }

        public void DeleteCustomer(Customer toRemove)
        {
            if (Customers.Contains(toRemove))
            {
                Customers.Remove(toRemove);
            }
        }

        public void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}