﻿using System;
using System.Collections.Generic;
using SampleMvvm.CustomerWcfService;

namespace SampleMvvm.Model
{
    public class CustomerServiceProxy : ICustomerServiceProxy
    {
        private CustomerServiceClient _client;

        public void GetCustomers(Action<IEnumerable<Customer>, Exception> callback)
        {
            EnsureClient();
            _client.GetAllCustomersCompleted += (s, e) =>
            {
                if (e.Error != null)
                {
                    callback(null, e.Error);
                }
                else
                {
                    callback(e.Result, null);
                }
            };
            _client.GetAllCustomersAsync();
        }

        public void SaveCustomer(Customer c, Action<bool> callback)
        {
            EnsureClient();
            _client.SaveCustomerCompleted += (s, e) => callback(e.Result);
            _client.SaveCustomerAsync(c);
        }

        private void EnsureClient()
        {
            if (_client == null)
            {
                _client = new CustomerServiceClient("BasicEndPoint");
            }
        }
    }
}