﻿using System;
using System.Collections.Generic;
using SampleMvvm.CustomerWcfService;
using System.Collections.ObjectModel;

namespace SampleMvvm.Model
{
    public class CustomerServiceProxy : ICustomerServiceProxy
    {
        private CustomerServiceClient _client;

        public void GetCustomers(Action<IEnumerable<Customer>, Exception> callback)
        {
            EnsureClient();
            _client.GetAllCustomersCompleted += (s, e) =>
            {
                if (e.Error != null)
                {
                    callback(null, e.Error);
                }
                else
                {
                    callback(e.Result, null);
                }
            };
            _client.GetAllCustomersAsync();
        }

        public void SaveCustomers(IEnumerable<Customer> customers, Action<bool> callback)
        {
            EnsureClient();

            _client.SaveCustomersCompleted += (s, e) =>
            {
                if (e.Error != null)
                {
                    callback(false);
                }

                callback(e.Result);
            };

            var collection = new ObservableCollection<Customer>(customers);
            _client.SaveCustomersAsync(collection);
        }

        private void EnsureClient()
        {
            if (_client == null)
            {
                _client = new CustomerServiceClient("BasicEndPoint");
            }
        }
    }
}