﻿using System.Windows;
using SampleMvvm.CustomerWcfService;
using SampleMvvm.MvvmHelpers;

namespace SampleMvvm.ViewModel
{
    public class CustomerViewModel : ViewModelBase
    {
        public Customer Model
        {
            get;
            private set;
        }

        /// <summary>
        /// The <see cref="DirtyVisibility" /> property's name.
        /// </summary>
        public const string DirtyVisibilityPropertyName = "DirtyVisibility";

        private Visibility _dirty = Visibility.Collapsed;

        /// <summary>
        /// Gets the DirtyVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility DirtyVisibility
        {
            get
            {
                return _dirty;
            }

            set
            {
                if (_dirty == value)
                {
                    return;
                }

                _dirty = value;
                RaisePropertyChanged(DirtyVisibilityPropertyName);
            }
        }

        public CustomerViewModel(Customer model)
        {
            Model = model;
            model.PropertyChanged += (s, e) =>
            {
                DirtyVisibility = Visibility.Visible;
            };
        }
    }
}
