﻿using System;
using System.ComponentModel.DataAnnotations;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace SimpleDataGrid.Data
{
    public class DataItem : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public const string NamePropertyName = "Name";

        private string _name = string.Empty;

        public string Name
        {
            get
            {
                return _name;
            }

            set
            {
                if (_name == value)
                {
                    return;
                }

                Validate(value, NamePropertyName);
                _name = value;
                RaisePropertyChanged(NamePropertyName);
            }
        }

        public const string ContractNumberPropertyName = "ContractNumber";

        private int _contractNumber;

        [Display(
            Name = "ContractNumberColumn",
            Description = "DescriptionColumn",
            ResourceType = typeof(Properties.LocalizableTexts))]
        [Range(10000, 99999,
            ErrorMessageResourceName = "ContractNumberRangeError",
            ErrorMessageResourceType = typeof(Properties.LocalizableTexts))]
        public int ContractNumber
        {
            get
            {
                return _contractNumber;
            }

            set
            {
                if (_contractNumber == value)
                {
                    return;
                }

                Validate(value, ContractNumberPropertyName);
                _contractNumber = value;
                RaisePropertyChanged(ContractNumberPropertyName);
            }
        }

        public const string IsDirtyPropertyName = "IsDirty";

        private bool _isDirty;

        [Display(AutoGenerateField = false)]
        public bool IsDirty
        {
            get
            {
                return _isDirty;
            }

            set
            {
                if (_isDirty == value)
                {
                    return;
                }

                _isDirty = value;
                RaisePropertyChanged(IsDirtyPropertyName);
            }
        }

        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void Validate(object value, string propertyName)
        {
            Validator.ValidateProperty(value,
                new ValidationContext(this, null, null)
                {
                    MemberName = propertyName
                });
        }
    }
}
