﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Net.NetworkInformation;

namespace TestingNetwork
{
    // Offline
    // Online
    public enum State
    {
        Unknown,
        OnlineInBrowserNormal,
        OnlineOutOfBrowserNormal,
        OnlineOutOfBrowserElevated,
        OfflineInBrowserNormal,
        OfflineOutOfBrowserNormal,
        OfflineOutOfBrowserElevated
    }

    public partial class MainPage : UserControl
    {
        /// <summary>
        /// The <see cref="ApplicationState" /> dependency property's name.
        /// </summary>
        public const string ApplicationStatePropertyName = "ApplicationState";

        /// <summary>
        /// Gets or sets the value of the <see cref="ApplicationState" />
        /// property. This is a dependency property.
        /// </summary>
        public State ApplicationState
        {
            get
            {
                return (State)GetValue(ApplicationStateProperty);
            }
            set
            {
                SetValue(ApplicationStateProperty, value);
            }
        }

        /// <summary>
        /// Identifies the <see cref="ApplicationState" /> dependency property.
        /// </summary>
        public static readonly DependencyProperty ApplicationStateProperty = DependencyProperty.Register(
            ApplicationStatePropertyName,
            typeof(State),
            typeof(MainPage),
            new PropertyMetadata(State.Unknown));

        public MainPage()
        {
            InitializeComponent();

            NetworkChange.NetworkAddressChanged += NetworkChanged;

            CheckApplicationState();
        }

        private void NetworkChanged(object sender, EventArgs e)
        {
            CheckApplicationState();
        }

        private void CheckApplicationState()
        {
            if (NetworkInterface.GetIsNetworkAvailable())
            {
                if (App.Current.IsRunningOutOfBrowser)
                {
                    if (App.Current.HasElevatedPermissions)
                    {
                        ApplicationState = State.OnlineOutOfBrowserElevated;
                    }
                    else
                    {
                        ApplicationState = State.OnlineOutOfBrowserNormal;
                    }
                }
                else
                {
                    ApplicationState = State.OnlineInBrowserNormal;
                }
            }
            else
            {
                if (App.Current.IsRunningOutOfBrowser)
                {
                    if (App.Current.HasElevatedPermissions)
                    {
                        ApplicationState = State.OfflineOutOfBrowserElevated;
                    }
                    else
                    {
                        ApplicationState = State.OfflineOutOfBrowserNormal;
                    }
                }
                else
                {
                    ApplicationState = State.OfflineInBrowserNormal;
                }
            }
        }
    }
}
