﻿using System;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using TextBoxHandling.Model;

namespace TextBoxHandling.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        public const string WaitAnimationName = "WaitAnimation";

        public const string CurrentCustomerPropertyName = "CurrentCustomer";
        private CustomerViewModel _customer = null;

        public CustomerViewModel CurrentCustomer
        {
            get
            {
                return _customer;
            }

            set
            {
                if (_customer == value)
                {
                    return;
                }

                _customer = value;
                RaisePropertyChanged(CurrentCustomerPropertyName);
            }
        }

        public IAnimationService AnimationService
        {
            get;
            set;
        }

        public RelayCommand CheckAndSaveCommand
        {
            get;
            private set;
        }

        private bool _isSaving;
        public bool IsSaving
        {
            get { return _isSaving; }

            set
            {
                if (_isSaving == value)
                {
                    return;
                }

                _isSaving = value;
                CheckAndSaveCommand.RaiseCanExecuteChanged();

                if (AnimationService != null)
                {
                    if (_isSaving)
                    {
                        AnimationService.StartAnimation(WaitAnimationName);
                    }
                    else
                    {
                        AnimationService.StopAnimation(WaitAnimationName);
                    }
                }
            }
        }

        private readonly INetworkService _networkService;

        public MainViewModel(INetworkService networkService)
        {
            _networkService = networkService;
            CurrentCustomer = new CustomerViewModel();

            CheckAndSaveCommand = new RelayCommand(
                CheckAndSave,
                () => !IsSaving);
        }

        private void CheckAndSave()
        {
            if (CurrentCustomer != null
                && !string.IsNullOrEmpty(CurrentCustomer.FirstName)
                && !string.IsNullOrEmpty(CurrentCustomer.LastName))
            {
                IsSaving = true;
                _networkService.Save(SaveCompleted);
            }
        }

        private void SaveCompleted(bool sucess)
        {
            IsSaving = false;
        }

    }
}