﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace VirtualizationSample.ViewModel
{
    public class MainViewModel : INotifyPropertyChanged
    {
        private const int NumberOfItems = 1000;

        private readonly DateTime _start;

        public const string ItemsPropertyName = "Items";
        private ObservableCollection<Item> _items = null;
        public ObservableCollection<Item> Items
        {
            get
            {
                return _items;
            }

            set
            {
                if (_items == value)
                {
                    return;
                }

                _items = value;
                RaisePropertyChanged(ItemsPropertyName);
            }
        }

        public MainViewModel()
        {
            _start = DateTime.Now;
            WriteDebugMessage("ViewModel created, creating items");

            var tempCollection = new ObservableCollection<Item>();

            for (var index = 0; index < NumberOfItems; index++)
            {
                var item = new Item
                {
                    Message = "This is item #" + index
                };

                tempCollection.Add(item);
            }

            WriteDebugMessage("Items created, assigning ObservableCollection");

            Items = tempCollection;

            WriteDebugMessage("Assigned");
        }

        [Conditional("DEBUG")]
        public void WriteDebugMessage(string message)
        {
            var span = DateTime.Now - _start;
            Debug.WriteLine(
                "{0:D2}:{1:D2}:{2:D3} - {3}",
                span.Minutes,
                span.Seconds,
                span.Milliseconds,
                message);
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class Item
    {
        public string Message { get; set; }
    }

}
