﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Threading;
using System.Windows.Threading;
using VirtualizationSample.Services;
using System.Linq;

namespace VirtualizationSample.ViewModel
{
    public class MainViewModel : INotifyPropertyChanged
    {
        private const int NumberOfItems = 10000;

        public const string ItemsPropertyName = "Items";
        private ObservableCollection<Item> _items = null;
        public ObservableCollection<Item> Items
        {
            get
            {
                return _items;
            }

            set
            {
                if (_items == value)
                {
                    return;
                }

                _items = value;
                RaisePropertyChanged(ItemsPropertyName);
            }
        }

        private const int BatchSize = 100;
        private readonly Dispatcher _dispatcher;

        public MainViewModel()
        {
            Items = new ObservableCollection<Item>();
            _dispatcher = Deployment.Current.Dispatcher;
            ThreadPool.QueueUserWorkItem(CreateItems);
        }

        private void CreateItems(object state)
        {
            // This method runs in a background thread!

            int itemsCounter = 0;

            while (itemsCounter < NumberOfItems)
            {
                IEnumerable<Item> batchItems;
                if (itemsCounter + BatchSize < NumberOfItems)
                {
                    batchItems = ItemsService.GetItems(BatchSize);
                }
                else
                {
                    batchItems = ItemsService.GetItems(
                        NumberOfItems - itemsCounter);
                }

                foreach (var item in batchItems)
                {
                    _dispatcher.BeginInvoke(() => Items.Add(item));
                }

                itemsCounter += batchItems.Count();
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class Item
    {
        public string Message { get; set; }
    }

}
