﻿using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO.IsolatedStorage;

namespace WebBrowserSample.Helpers
{
    public class LocalFileHelper
    {
        private const string IsContentLocalKey = "IsContentLocal";
        private const string LastAddressKey = "LastAddress";
        private const string SettingsFileName = "__LocalSettings";

        public string FileContent
        {
            get;
            set;
        }

        public bool SaveFile(string fileName)
        {
            if (string.IsNullOrEmpty(fileName)
                || string.IsNullOrEmpty(FileContent))
            {
                return false;
            }

            using (var isoStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isoStore.AvailableFreeSpace < FileContent.Length)
                {
                    if (!isoStore.IncreaseQuotaTo(isoStore.Quota + FileContent.Length * 2))
                    {
                        return false;
                    }
                }

                using (var stream = isoStore.OpenFile(fileName, FileMode.CreateNew))
                {
                    using (var writer = new StreamWriter(stream))
                    {
                        writer.Write(FileContent);
                    }
                }
            }

            return true;
        }

        public string[] GetAvailableFiles()
        {
            using (var isoStore
                = IsolatedStorageFile.GetUserStoreForApplication())
            {
                return isoStore.GetFileNames().ToList()
                    .Where(n => n != SettingsFileName)
                    .ToArray();
            }
        }

        public bool FileExists(string fileName)
        {
            var list = GetAvailableFiles().ToList();

            return list.Contains(fileName);
        }

        public bool GetFileContent(string fileName)
        {
            if (!FileExists(fileName))
            {
                return false;
            }

            using (var isoStore 
                = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (var stream = isoStore.OpenFile(fileName, FileMode.Open))
                {
                    using (var reader = new StreamReader(stream))
                    {
                        FileContent = reader.ReadToEnd();
                    }
                }
            }

            return true;
        }

        public void DeleteFile(string fileName)
        {
            if (FileExists(fileName))
            {
                using (var isoStore
                    = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    isoStore.DeleteFile(fileName);
                }
            }
        }

        public void SaveSettings(bool isContentLocal, string lastAddress)
        {
            IsolatedStorageSettings.ApplicationSettings[IsContentLocalKey]
                = isContentLocal;
            IsolatedStorageSettings.ApplicationSettings[LastAddressKey]
                = lastAddress;
        }

        public void LoadSettings(out bool isContentLocal, out string lastAddress)
        {
            isContentLocal = false;
            lastAddress = string.Empty;

            if (IsolatedStorageSettings.ApplicationSettings.Contains(LastAddressKey)
                && IsolatedStorageSettings.ApplicationSettings.Contains(IsContentLocalKey))
            {
                lastAddress = IsolatedStorageSettings.ApplicationSettings[LastAddressKey].ToString();
                isContentLocal = (bool)IsolatedStorageSettings.ApplicationSettings[IsContentLocalKey];
            }
        }
    }
}
