﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections.ObjectModel;
using WebBrowserSample.Helpers;

namespace WebBrowserSample
{
    public partial class LoadFileChildWindow : ChildWindow
    {
        private LocalFileHelper _helper = new LocalFileHelper();

        public string FileContent
        {
            get { return _helper.FileContent; }
        }

        public string FileName
        {
            get
            {
                if (FilesListBox.SelectedIndex < 0)
                {
                    return string.Empty;
                }

                return FilesListBox.SelectedItem.ToString();
            }
        }

        public ObservableCollection<string> Files
        {
            get;
            private set;
        }

        public LoadFileChildWindow()
        {
            Files = new ObservableCollection<string>(_helper.GetAvailableFiles());
            InitializeComponent();
        }

        private void OKButton_Click(object sender, RoutedEventArgs e)
        {
            if (FilesListBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a file or cancel");
                return;
            }

            if (!_helper.GetFileContent(
                FilesListBox.SelectedItem.ToString()))
            {
                MessageBox.Show("Cannot get file content, try again");
                return;
            }

            DialogResult = true;
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = false;
        }

        private void DeleteFile_Click(object sender, RoutedEventArgs e)
        {
            if (FilesListBox.SelectedIndex < 0)
            {
                return;
            }

            var fileName = FilesListBox.SelectedItem.ToString();
            _helper.DeleteFile(fileName);
            Files.Remove(fileName);
        }
    }
}

