﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO.IsolatedStorage;
using System.Security;
using WebBrowserSample.Helpers;

namespace WebBrowserSample
{
    public partial class MainPage : UserControl
    {
        private bool _isContentLocal;
        private string _lastAddress = string.Empty;
        private LocalFileHelper _helper = new LocalFileHelper();

        public MainPage()
        {
            InitializeComponent();

            MyWebBrowser.LoadCompleted += (s, e) => 
                MyBusyIndicator.IsBusy = false;

            Application.Current.Exit += (s, e) => 
                _helper.SaveSettings(_isContentLocal, _lastAddress);

            LoadInitialPage();
        }

        private void NavigateButtonClick(object sender, RoutedEventArgs e)
        {
            Navigate();
        }

        private void Navigate()
        {
            if (string.IsNullOrEmpty(LocationTextBox.Text))
            {
                return;
            }

            MyBusyIndicator.IsBusy = true;
            MyWebBrowser.Source = GetNavigationUri();
            _lastAddress = MyWebBrowser.Source.AbsoluteUri;
            _isContentLocal = false;
        }

        private Uri GetNavigationUri()
        {
            Uri nextUri;

            if (LocationTextBox.Text.StartsWith("http://"))
            {
                nextUri = new Uri(LocationTextBox.Text, UriKind.Absolute);
            }
            else
            {
                var xapUri = App.Current.Host.Source ;
                var baseAddress = xapUri.AbsoluteUri.Substring(
                    0, xapUri.AbsoluteUri.IndexOf("ClientBin/"));

                nextUri = new Uri(baseAddress
                    + LocationTextBox.Text, UriKind.Absolute);
            }

            return nextUri;
        }

        private void SaveFileButtonClick(object sender, RoutedEventArgs e)
        {
            try
            {
                var content = MyWebBrowser.SaveToString();
                var window = new SaveFileChildWindow(content);
                window.Closed += (s, args) => MyWebBrowser.Visibility = Visibility.Visible;
                window.Show();

                MyWebBrowserBrush.Redraw();
                MyWebBrowser.Visibility = Visibility.Collapsed;
            }
            catch (SecurityException)
            {
                MessageBox.Show("Impossible to save, no permission");
            }
        }

        private void LoadFileButtonClick(object sender, RoutedEventArgs e)
        {
            var window = new LoadFileChildWindow();
            window.Closing += (s, args) =>
            {
                if (window.DialogResult == true)
                {
                    MyWebBrowser.NavigateToString(window.FileContent);
                    _lastAddress = LocationTextBox.Text = window.FileName;
                    _isContentLocal = true;
                }

                MyWebBrowser.Visibility = Visibility.Visible;
            };

            MyWebBrowserBrush.Redraw();
            MyWebBrowser.Visibility = Visibility.Collapsed;
            window.Show();
        }

        private void FullScreenButtonClick(object sender, RoutedEventArgs e)
        {
            App.Current.Host.Content.IsFullScreen = true;
        }

        private void ShowHtmlButtonClick(object sender, RoutedEventArgs e)
        {
            MyWebBrowser.NavigateToString(LocationTextBox.Text);
        }

        private void LoadInitialPage()
        {
            _helper.LoadSettings(out _isContentLocal, out _lastAddress);

            if (!string.IsNullOrEmpty(_lastAddress))
            {
                LocationTextBox.Text = _lastAddress;

                if (_isContentLocal)
                {
                    if (_helper.GetFileContent(_lastAddress))
                    {
                        MyWebBrowser.NavigateToString(_helper.FileContent);
                    }
                }
                else
                {
                    Navigate();
                }
            }
        }

        private void HandleDrop(object sender, DragEventArgs e)
        {
            var files = e.Data.GetData(DataFormats.FileDrop) as FileInfo[];

            if (files == null || files.Length == 0)
            {
                return;
            }

            var isFirst = true;    
            foreach (var file in files)
            {
                using (var reader = file.OpenText())
                {
                    _helper.FileContent = reader.ReadToEnd();
                    _helper.SaveFile(file.Name);

                    if (isFirst)
                    {
                        MyWebBrowser.NavigateToString(_helper.FileContent);
                        LocationTextBox.Text = file.Name;
                        isFirst = false;
                    }
                }
            }
            MessageBox.Show(string.Format("{0} files saved in storage", files.Length));
        }
    }
}
