﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace WebServerMonitor
{
    public partial class MainPage : UserControl
    {
        private const int PingDelay = 5000; // milliseconds
        private const string PingFileName = "/ping.txt";
        private Uri _pingFileUri;

        public ObservableCollection<string> Entries
        {
            get;
            private set;
        }

        public BackgroundWorker Worker
        {
            get;
            private set;
        }

        public MainPage()
        {
            Entries = new ObservableCollection<string>();

            InitializeComponent();

            var source = Application.Current.Host.Source;
            var baseAddress = source.AbsoluteUri.Substring(
                0, source.AbsoluteUri.Length - source.AbsolutePath.Length);
            _pingFileUri = new Uri(baseAddress + PingFileName);

            StartMonitoring();
        }

        private void StartMonitoring()
        {
            Worker = new BackgroundWorker
            {
                WorkerReportsProgress = true
            };

            Worker.DoWork += (s, e) =>
            {
                var client = new WebClient();
                client.DownloadStringCompleted 
                    += client_DownloadStringCompleted;

                while (true)
                {
                    Worker.ReportProgress(0, 
                        "Pinging at " + DateTime.Now.ToString());
                    client.DownloadStringAsync(_pingFileUri);
                    Thread.Sleep(PingDelay);
                }
            };

            Worker.ProgressChanged += (s, e) => 
                Entries.Add(e.UserState.ToString());

            Worker.RunWorkerAsync();
        }

        void client_DownloadStringCompleted(
            object sender, 
            DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                Worker.ReportProgress(0, "Ping OK");
            }
            else
            {
                Worker.ReportProgress(0, 
                    "Ping Error: " + e.Error.Message);
            }
        }
    }
}
