﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using WhyDependencyInjection.Model;

namespace WhyDependencyInjection.ViewModel
{
    public class ViewModelWithDependencyInjection : INotifyPropertyChanged
    {
        public const string ItemsPropertyName = "Items";
        private ObservableCollection<Item> _items;
        public ObservableCollection<Item> Items
        {
            get
            {
                return _items;
            }

            set
            {
                if (_items == value)
                {
                    return;
                }

                _items = value;
                RaisePropertyChanged(ItemsPropertyName);
            }
        }

        public ViewModelWithDependencyInjection(IDataService service)
        {
            if (service == null)
            {
                return;
            }

            service.GetItems(items =>
            {
                if (items == null)
                {
                    Items = new ObservableCollection<Item>();
                    return;
                }

                Items = new ObservableCollection<Item>(items);
            });
        }
    
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
